#ifndef TrackingMonitorClient_TrackingCertificationInfo_h
#define TrackingMonitorClient_TrackingCertificationInfo_h
// -*- C++ -*-
//
// Package:     TrackingMonitorClient
// Class  :     TrackingCertificationInfo
// 
/**\class TrackingCertificationInfo TrackingCertificationInfo.h DQM/TrackingMonitorClient/interface/TrackingCertificationInfo.h

 Description: 

 Usage:
    <usage>

*/

#include <string>

#include "DQMServices/Core/interface/DQMEDHarvester.h"
#include "FWCore/ParameterSet/interface/ParameterSet.h"
#include "FWCore/Framework/interface/ESHandle.h"
#include "FWCore/Framework/interface/LuminosityBlock.h"
#include "FWCore/Framework/interface/Run.h"
#include "FWCore/Framework/interface/EventSetup.h"

#include "DQMServices/Core/interface/DQMStore.h"

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>

class MonitorElement;
class SiStripDetCabling;

class TrackingCertificationInfo: public DQMEDHarvester
{

 public:

  /// Constructor
  TrackingCertificationInfo(const edm::ParameterSet& ps);
  
  /// Destructor
  virtual ~TrackingCertificationInfo();

 private:

  /// BeginJob
  void beginJob();

  /// Begin Run
  void beginRun(edm::Run const& run, edm::EventSetup const& eSetup);

  /// End Of Luminosity
  void dqmEndLuminosityBlock(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_,edm::LuminosityBlock const& lumiSeg, edm::EventSetup const& iSetup);
  
  /// EndJob
  void dqmEndJob(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);

private:

  void bookTrackingCertificationMEs(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);
  void bookTrackingCertificationMEsAtLumi(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);

  void resetTrackingCertificationMEs(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);
  void resetTrackingCertificationMEsAtLumi(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);

  void fillTrackingCertificationMEs(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);
  void fillTrackingCertificationMEsAtLumi(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);

  void fillDummyTrackingCertification(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);
  void fillDummyTrackingCertificationAtLumi(DQMStore::IBooker & ibooker_, DQMStore::IGetter & igetter_);


  struct TrackingMEs{
    MonitorElement* TrackingFlag;
  };

  struct TrackingLSMEs{
    MonitorElement* TrackingFlag;
  };

  std::map<std::string, TrackingMEs>   TrackingMEsMap;
  std::map<std::string, TrackingLSMEs> TrackingLSMEsMap;

  MonitorElement * TrackingCertification;  
  MonitorElement * TrackingCertificationSummaryMap;  

  MonitorElement * TrackingLSCertification;  

  edm::ESHandle< SiStripDetCabling > detCabling_;
  edm::ParameterSet pSet_;

  bool trackingCertificationBooked_;
  bool trackingLSCertificationBooked_;
  int nFEDConnected_;
  bool allPixelFEDConnected_;
  bool verbose_;
  std::string TopFolderName_;

  bool checkPixelFEDs_;

  unsigned long long m_cacheID_;

  std::vector<std::string> SubDetFolder;
};
#endif
